@@ -43,7 +43,6 @@ pub(crate) struct SourceAnalyzer {
     body: Option<Arc<Body>>,
     body_source_map: Option<Arc<BodySourceMap>>,
     infer: Option<Arc<InferenceResult>>,
-    scopes: Option<Arc<ExprScopes>>,
 }
 
 impl SourceAnalyzer {
@@ -65,7 +64,6 @@ impl SourceAnalyzer {
             body: Some(body),
             body_source_map: Some(source_map),
             infer: Some(db.infer(def)),
-            scopes: Some(scopes),
             file_id: node.file_id,
         }
     }
@@ -79,7 +77,6 @@ impl SourceAnalyzer {
             body: None,
             body_source_map: None,
             infer: None,
-            scopes: None,
             file_id: node.file_id,
         }
     }
@@ -116,46 +113,36 @@ impl SourceAnalyzer {
         Some(res)
     }
 
-    pub(crate) fn type_of_expr(&self, db: &dyn HirDatabase, expr: &ast::Expr) -> Option<Type> {
-        let expr_id = self.expr_id(db, expr)?;
-        let ty = self.infer.as_ref()?[expr_id].clone();
-        Type::new_with_resolver(db, &self.resolver, ty)
-    }
-
-    pub(crate) fn type_of_expr_with_coercion(
+    pub(crate) fn type_of_expr(
         &self,
         db: &dyn HirDatabase,
         expr: &ast::Expr,
-    ) -> Option<(Type, bool)> {
+    ) -> Option<(Type, Option<Type>)> {
         let expr_id = self.expr_id(db, expr)?;
         let infer = self.infer.as_ref()?;
-        let (ty, coerced) = infer
+        let coerced = infer
             .expr_adjustments
             .get(&expr_id)
-            .and_then(|adjusts| adjusts.last().map(|adjust| (&adjust.target, true)))
-            .unwrap_or_else(|| (&infer[expr_id], false));
-        Type::new_with_resolver(db, &self.resolver, ty.clone()).zip(Some(coerced))
+            .and_then(|adjusts| adjusts.last().map(|adjust| adjust.target.clone()));
+        let ty = infer[expr_id].clone();
+        let mk_ty = |ty| Type::new_with_resolver(db, &self.resolver, ty);
+        mk_ty(ty).zip(Some(coerced.and_then(mk_ty)))
     }
 
-    pub(crate) fn type_of_pat(&self, db: &dyn HirDatabase, pat: &ast::Pat) -> Option<Type> {
-        let pat_id = self.pat_id(pat)?;
-        let ty = self.infer.as_ref()?[pat_id].clone();
-        Type::new_with_resolver(db, &self.resolver, ty)
-    }
-
-    pub(crate) fn type_of_pat_with_coercion(
+    pub(crate) fn type_of_pat(
         &self,
         db: &dyn HirDatabase,
         pat: &ast::Pat,
-    ) -> Option<Type> {
+    ) -> Option<(Type, Option<Type>)> {
         let pat_id = self.pat_id(pat)?;
         let infer = self.infer.as_ref()?;
-        let ty = infer
+        let coerced = infer
             .pat_adjustments
             .get(&pat_id)
-            .and_then(|adjusts| adjusts.last().map(|adjust| &adjust.target))
-            .unwrap_or_else(|| &infer[pat_id]);
-        Type::new_with_resolver(db, &self.resolver, ty.clone())
+            .and_then(|adjusts| adjusts.last().map(|adjust| adjust.target.clone()));
+        let ty = infer[pat_id].clone();
+        let mk_ty = |ty| Type::new_with_resolver(db, &self.resolver, ty);
+        mk_ty(ty).zip(Some(coerced.and_then(mk_ty)))
     }
 
     pub(crate) fn type_of_self(
@@ -259,6 +246,18 @@ impl SourceAnalyzer {
         }
     }
 
+    pub(crate) fn resolve_path_as_macro(
+        &self,
+        db: &dyn HirDatabase,
+        path: &ast::Path,
+    ) -> Option<MacroDef> {
+        // This must be a normal source file rather than macro file.
+        let hygiene = Hygiene::new(db.upcast(), self.file_id);
+        let ctx = body::LowerCtx::with_hygiene(db.upcast(), &hygiene);
+        let hir_path = Path::from_src(path.clone(), &ctx)?;
+        resolve_hir_path_as_macro(db, &self.resolver, &hir_path)
+    }
+
     pub(crate) fn resolve_path(
         &self,
         db: &dyn HirDatabase,
@@ -515,6 +514,15 @@ pub(crate) fn resolve_hir_path(
     resolve_hir_path_(db, resolver, path, false)
 }
 
+#[inline]
+pub(crate) fn resolve_hir_path_as_macro(
+    db: &dyn HirDatabase,
+    resolver: &Resolver,
+    path: &Path,
+) -> Option<MacroDef> {
+    resolver.resolve_path_as_macro(db.upcast(), path.mod_path()).map(Into::into)
+}
+
 fn resolve_hir_path_(
     db: &dyn HirDatabase,
     resolver: &Resolver,
